﻿using System.Windows.Forms;
using System.Drawing;

namespace Orator.Sample.CS
{
    public partial class Form1 : Form
    {
        private WBMessageController wbc;
        public Form1()
        {
            InitializeComponent();
            comboBox1.SelectedIndex = 0;
        }

        void wbc_ShowMessage(object sender, WBMessageController.ShowMessageEventArgs e)
        {
            // alert や confirm の呼び出し時に処理されるイベントです。
            // VBScript の MsgBox 関数呼び出し時にも反応します。

            switch (comboBox1.SelectedIndex)
            {
                case 1:
                    // 常に [OK] とする場合
                    e.Result = DialogResult.OK;
                    e.Handled = true;
                    break;
                case 2:
                    // 常に [キャンセル] とする場合
                    e.Result = DialogResult.Cancel;
                    e.Handled = true;
                    break;
                case 3:
                    // メッセージボックスを自分で呼びなおす事もできる
                    // 引数定義は、MessageBox API を参照
                    // (VB の MsgBox 関数とほぼ同じ)
                    e.Result = WBMessageController.MessageBox(
                            e.Handle,
                            "☆MessageBox API★\r\n" + e.Text,
                            e.Caption,
                            e.Type);
                    e.Handled = true;
                    break;
                case 4:
                    // MessageBox.Show を使う場合は、
                    // ビットフラグを分ける必要がある
                    e.Result = MessageBox.Show(
                            this,
                            "★MessageBox.Show☆\r\n" + e.Text,
                            e.Caption,
                            (MessageBoxButtons)(e.Type & 0xFu),
                            (MessageBoxIcon)(e.Type & 0xF0u),
                            (MessageBoxDefaultButton)(e.Type & 0xF00u),
                            (MessageBoxOptions)(e.Type & 0xF0000u));

                    e.Handled = true;
                    break;
                default:
                    // e.Handled の既定値は false 。
                    // false をセットしておいた場合には、
                    // WebBrowser の標準のメッセージで処理され、
                    // true をセットしたおいた場合には、
                    // メッセージ処理が抑制される。
                    e.Handled = false;
                    break;
            }
        }

        void wbc_ShowHelp(object sender, WBMessageController.ShowHelpEventArgs e)
        {
            // ヘルプ表示(F1キーなど)の場合、このイベントが呼び出される。
            e.Handled = false;

            // ヘルプ表示を自分で行う場合の例。
            // Point pt = e.Mouse;
             WBMessageController.WinHelp(e.Handle, e.HelpFile, e.Command, e.Data);
            // e.Handled = false;
        }

        private void Form1_Load(object sender, System.EventArgs e)
        {
            this.wbc = new WBMessageController(this.webBrowser1.ActiveXInstance);
            this.wbc.ShowHelp += wbc_ShowHelp;
            this.wbc.ShowMessage += wbc_ShowMessage;

            this.webBrowser1.Navigate("http://www.tagindex.com/javascript/window/confirm.html");
            //this.webBrowser1.Navigate("http://www.red.oit-net.jp/tatsuya/java/confirm.htm");
        }
    }
}
