﻿using System.Runtime.InteropServices;
using System.Windows.Forms;
using System;
using System.Drawing;
using System.Runtime.CompilerServices;

[ComVisible(true)]
public class WBMessageController : IOleClientSite, IDocHostShowUI
{
    #region イベント
    public event EventHandler<ShowHelpEventArgs> ShowHelp;
    public event EventHandler<ShowMessageEventArgs> ShowMessage;

    protected virtual void OnShowHelp(ShowHelpEventArgs e)
    {
        var handler = this.ShowHelp;
        if (handler != null) handler(this, e);
    }

    protected virtual void OnShowMessage(ShowMessageEventArgs e)
    {
        var handler = this.ShowMessage;
        if (handler != null) handler(this, e);
    }
    #endregion

    public WBMessageController(object axWebBrowser)
    {
        ((IOleObject)axWebBrowser).SetClientSite(this);
    }

    #region IOleClientSite
    void IOleClientSite.SaveObject() { }
    void IOleClientSite.GetMoniker(int dwAssign, int dwWhichMoniker, ref object ppmk) { }
    void IOleClientSite.GetContainer(ref object ppContainer) { }
    void IOleClientSite.ShowObject() { }
    void IOleClientSite.OnShowWindow(bool fShow) { }
    void IOleClientSite.RequestNewObjectLayout() { }
    #endregion

    #region IDocHostShowUI
    int IDocHostShowUI.ShowMessage(IntPtr hwnd, string lpstrText, string lpstrCaption, uint dwType, string lpstrHelpFile, uint dwHelpContext, ref uint lpResult)
    {
        DialogResult dr = (DialogResult)lpResult;
        var e = new ShowMessageEventArgs(hwnd, lpstrText, lpstrCaption, dwType, lpstrHelpFile, dwHelpContext, ref dr);
        OnShowMessage(e);
        if (e.Handled)
        {
            lpResult = (uint)e.Result;
            return /* S_OK */ 0;
        }
        else
        {
            return /* S_FALSE */ 1;
        }
    }
    
    int IDocHostShowUI.ShowHelp(IntPtr hwnd, string pszHelpFile, uint uCommand, uint dwData, POINT ptMouse, out object pDispatchObjectHit)
    {
        Point point = new Point(ptMouse.x, ptMouse.y);
        var e = new ShowHelpEventArgs(hwnd, pszHelpFile, uCommand, dwData, point);
        OnShowHelp(e);
        pDispatchObjectHit = e.DispatchObjectHit;
        return e.Handled ? /* S_OK    */ 0
                         : /* S_FALSE */ 1;
    }
    #endregion

    #region イベントハンドラ ShowHelpEventArgs
    public class ShowHelpEventArgs : EventArgs
    {
        public bool Handled { get; set; }

        private IntPtr handle;
        public IntPtr Handle { get { return handle; } }

        private string helpFile;
        public string HelpFile { get { return helpFile; } }

        private uint command;
        public uint Command { get { return command; } }

        private uint data;
        public uint Data { get { return data; } }

        private Point mouse;
        public Point Mouse { get { return mouse; } }

        public object DispatchObjectHit { get; set; }

        public ShowHelpEventArgs(IntPtr hwnd, string pszHelpFile, uint uCommand, uint dwData, Point ptMouse)
        {
            this.handle = hwnd;
            this.helpFile = pszHelpFile;
            this.command = uCommand;
            this.data = dwData;
            this.mouse = ptMouse;
        }
    }
    #endregion

    #region イベント ハンドラ ShowMessageEventArgs
    public class ShowMessageEventArgs : EventArgs
    {
        public bool Handled { get; set; }

        private IntPtr handle;
        public IntPtr Handle { get { return handle; } }

        private string text;
        public string Text { get { return text; } }

        private string caption;
        public string Caption { get { return caption; } }

        private uint type;
        public uint Type { get { return type; } }

        private string helpFile;
        public string HelpFile { get { return helpFile; } }

        private uint helpContext;
        public uint HelpContext { get { return helpContext; } }

        public DialogResult Result { get; set; }

        public ShowMessageEventArgs(IntPtr hwnd, string lpstrText, string lpstrCaption, uint dwType, string lpstrHelpFile, uint dwHelpContext, ref DialogResult plResult)
        {
            this.Handled = false;

            this.handle = hwnd;
            this.text = lpstrText;
            this.caption = lpstrCaption;
            this.type = dwType;
            this.helpFile = lpstrHelpFile;
            this.helpContext = dwHelpContext;

            this.Result = plResult;
        }
    }
    #endregion

    [DllImport("user32.dll", CharSet = CharSet.Auto)]
    public static extern DialogResult MessageBox(
        IntPtr handle,
        string text,
        String caption,
        uint type);

    [DllImport("user32.dll", CharSet = CharSet.Auto)]
    public static extern bool WinHelp(
        IntPtr hWndMain,
        string lpszHelp,
        uint uCommand,
        uint dwData);
}


public struct POINT { public int x, y; }

#region IOleClientSite
[ComImport]
[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
[Guid("00000118-0000-0000-C000-000000000046")]
public interface IOleClientSite
{
    void SaveObject();
    void GetMoniker(int dwAssign, int dwWhichMoniker, ref object ppmk);
    void GetContainer(ref object ppContainer);
    void ShowObject();
    void OnShowWindow(bool fShow);
    void RequestNewObjectLayout();
}
#endregion

#region IDocHostShowUI
[ComImport]
[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
[Guid("C4D244B0-D43E-11CF-893B-00AA00BDCE1A"), ComVisible(true)]
public interface IDocHostShowUI
{
    [return: MarshalAs(UnmanagedType.I4)]
    [PreserveSig]
    int ShowMessage(IntPtr hwnd, [MarshalAs(UnmanagedType.LPWStr)] string lpstrText, [MarshalAs(UnmanagedType.LPWStr)] string lpstrCaption, [MarshalAs(UnmanagedType.U4)] uint dwType, [MarshalAs(UnmanagedType.LPWStr)] string lpstrHelpFile, [MarshalAs(UnmanagedType.U4)] uint dwHelpContext, [In, Out] ref uint lpResult);
    [return: MarshalAs(UnmanagedType.I4)]
    [PreserveSig]
    int ShowHelp(IntPtr hwnd, [MarshalAs(UnmanagedType.LPWStr)] string pszHelpFile, [MarshalAs(UnmanagedType.U4)] uint uCommand, [MarshalAs(UnmanagedType.U4)] uint dwData, [In, MarshalAs(UnmanagedType.Struct)] POINT ptMouse, [Out, MarshalAs(UnmanagedType.IDispatch)] out object pDispatchObjectHit);
}
#endregion

#region IOleObject
[ComImport]
[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
[Guid("00000112-0000-0000-C000-000000000046")]
public interface IOleObject
{
    void SetClientSite(IOleClientSite pClientSite);
    void GetClientSite(ref IOleClientSite ppClientSite);
    void SetHostNames(object szContainerApp, object szContainerObj);
    void Close(int dwSaveOption);
    void SetMoniker(int dwWhichMoniker, object pmk);
    void GetMoniker(int dwAssign, int dwWhichMoniker, object ppmk);
    void InitFromData(IDataObject pDataObject, bool fCreation, int dwReserved);
    void GetClipboardData(int dwReserved, ref IDataObject ppDataObject);
    void DoVerb(int iVerb, int lpmsg, object pActiveSite, int lindex, int hwndParent, int lprcPosRect);
    void EnumVerbs(ref object ppEnumOleVerb);
    void Update();
    void IsUpToDate();
    void GetUserClassID(int pClsid);
    void GetUserType(int dwFormOfType, int pszUserType);
    void SetExtent(int dwDrawAspect, int psizel);
    void GetExtent(int dwDrawAspect, int psizel);
    void Advise(object pAdvSink, int pdwConnection);
    void Unadvise(int dwConnection);
    void EnumAdvise(ref object ppenumAdvise);
    void GetMiscStatus(int dwAspect, int pdwStatus);
    void SetColorScheme(object pLogpal);
}
#endregion
